import * as React from 'react';
import { BarActionType, } from '../types';
import { withResizerContext } from '../context';
import { StyledBar, StyledInteractiveArea } from './Bar.styled';
import { disablePassive } from './disablePassive';
class BarComponent extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.defaultInnerRef = React.createRef();
        this.interactiveAreaRef = React.createRef();
        this.id = this.props.context.createID(this.props);
        this.isValidClick = true;
        this.isActivated = false;
        this.onMouseDown = this.triggerMouseAction(BarActionType.ACTIVATE);
        this.onMouseMove = this.triggerMouseAction(BarActionType.MOVE);
        this.onMouseUp = this.triggerMouseAction(BarActionType.DEACTIVATE);
        this.onTouchStart = this.triggerTouchAction(BarActionType.ACTIVATE);
        this.onTouchMove = this.triggerTouchAction(BarActionType.MOVE);
        this.onTouchEnd = this.triggerTouchAction(BarActionType.DEACTIVATE);
        this.onTouchCancel = this.triggerTouchAction(BarActionType.DEACTIVATE);
    }
    get ref() {
        return this.props.innerRef || this.defaultInnerRef;
    }
    componentDidMount() {
        this.props.context.populateInstance(this.id, this.ref);
        document.addEventListener('mousemove', this.onMouseMove);
        document.addEventListener('mouseup', this.onMouseUp);
        document.addEventListener('touchmove', this.onTouchMove, disablePassive);
        document.addEventListener('touchend', this.onTouchEnd);
        document.addEventListener('touchcancel', this.onTouchCancel);
        if (this.interactiveAreaRef.current) {
            this.interactiveAreaRef.current.addEventListener('mousedown', this.onMouseDown);
            this.interactiveAreaRef.current.addEventListener('touchstart', this.onTouchStart, disablePassive);
        }
    }
    componentWillUnmount() {
        document.removeEventListener('mousemove', this.onMouseMove);
        document.removeEventListener('mouseup', this.onMouseUp);
        document.removeEventListener('touchmove', this.onTouchMove);
        document.removeEventListener('touchend', this.onTouchEnd);
        document.removeEventListener('touchcancel', this.onTouchCancel);
        if (this.interactiveAreaRef.current) {
            this.interactiveAreaRef.current.removeEventListener('mousedown', this.onMouseDown);
            this.interactiveAreaRef.current.removeEventListener('touchstart', this.onTouchStart);
        }
    }
    render() {
        return (React.createElement(StyledBar, Object.assign({}, this.props, { ref: this.ref }),
            this.props.children,
            React.createElement(StyledInteractiveArea, Object.assign({}, this.props.expandInteractiveArea, { ref: this.interactiveAreaRef, vertical: this.props.context.vertical }))));
    }
    onStatusChanged(isActivated) {
        if (this.isActivated !== isActivated) {
            this.isActivated = isActivated;
            if (typeof this.props.onStatusChanged === 'function') {
                this.props.onStatusChanged(isActivated);
            }
        }
    }
    updateStatusIfNeed(type) {
        if (type === BarActionType.ACTIVATE) {
            this.onStatusChanged(true);
        }
        else if (type === BarActionType.DEACTIVATE) {
            this.onStatusChanged(false);
        }
    }
    triggerAction(type, coordinate) {
        if (this.isActivated || type === BarActionType.ACTIVATE) {
            this.props.context.triggerBarAction({ type, coordinate, barID: this.id });
        }
        if (this.isActivated && type === BarActionType.DEACTIVATE) {
            this.onClick();
        }
        this.updateStatusIfNeed(type);
        this.updateClickStatus(type);
    }
    triggerMouseAction(type) {
        return (event) => {
            this.disableUserSelectIfResizing(event, type);
            const { clientX: x, clientY: y } = event;
            this.triggerAction(type, { x, y });
        };
    }
    triggerTouchAction(type) {
        return (event) => {
            this.disableUserSelectIfResizing(event, type);
            const touch = event.touches[0] || { clientX: 0, clientY: 0 };
            const { clientX: x, clientY: y } = touch;
            this.triggerAction(type, { x, y });
        };
    }
    disableUserSelectIfResizing(event, type) {
        if (this.isActivated || type === BarActionType.ACTIVATE) {
            event.preventDefault();
        }
    }
    updateClickStatus(type) {
        if (this.isActivated) {
            if (type === BarActionType.ACTIVATE) {
                this.isValidClick = true;
            }
            else if (type === BarActionType.MOVE) {
                this.isValidClick = false;
            }
        }
    }
    onClick() {
        if (this.isValidClick && typeof this.props.onClick === 'function') {
            this.isValidClick = false;
            this.props.onClick();
        }
    }
}
export const Bar = withResizerContext(BarComponent);
