import * as React from 'react';
import { animationFrameScheduler, merge, Subject } from 'rxjs';
import { filter, map, observeOn, share, tap } from 'rxjs/operators';
import { BarActionType, } from '../types';
import { ResizerProvider } from '../context';
import { Resizer } from './Resizer';
import { scanBarAction } from './operators';
import { calculateCoordinateOffset, collectSizeRelatedInfo, isDisabledResponsive, isSolid, } from './utils';
import { StyledContainer } from './Container.styled';
class Container extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.childrenProps = [];
        this.childrenInstance = [];
        this.barActions$ = new Subject();
        this.sizeRelatedInfoAction$ = new Subject();
        this.sizeRelatedInfo$ = merge(this.sizeRelatedInfoAction$, this.barActions$.pipe(scanBarAction({
            calculateOffset: (current, original) => calculateCoordinateOffset(current, original)[this.axis],
            getSizeRelatedInfo: () => this.makeSizeInfos(),
        }), tap((scanResult) => this.monitorBarStatusChanges(scanResult)))).pipe(filter(({ discard }) => !discard), map((resizeResult) => {
            if (typeof this.props.beforeApplyResizer === 'function') {
                const resizer = new Resizer(resizeResult);
                this.props.beforeApplyResizer(resizer);
                return resizer.getResult();
            }
            else {
                return resizeResult;
            }
        }), filter(({ discard }) => !discard), observeOn(animationFrameScheduler), share());
        this.triggerBarAction = (action) => {
            this.barActions$.next(action);
        };
        this.createID = (childProps) => {
            this.childrenProps.push(childProps);
            return this.childrenProps.length - 1;
        };
        this.populateChildInstance = (id, ref) => {
            if (ref.current) {
                this.childrenInstance[id] = ref.current;
            }
        };
    }
    get axis() {
        return this.props.vertical ? 'y' : 'x';
    }
    get dimension() {
        return this.props.vertical ? 'height' : 'width';
    }
    get contextValue() {
        return {
            vertical: !!this.props.vertical,
            sizeRelatedInfo$: this.sizeRelatedInfo$,
            createID: this.createID,
            populateInstance: this.populateChildInstance,
            triggerBarAction: this.triggerBarAction,
        };
    }
    componentDidMount() {
        this.refreshSizeInfos();
    }
    render() {
        return (React.createElement(ResizerProvider, { value: this.contextValue },
            React.createElement(StyledContainer, Object.assign({}, this.props, { vertical: this.props.vertical }), this.props.children)));
    }
    getResizer() {
        return new Resizer(this.makeSizeInfos());
    }
    applyResizer(resizer) {
        this.sizeRelatedInfoAction$.next(resizer.getResult());
    }
    monitorBarStatusChanges({ type }) {
        switch (type) {
            case BarActionType.ACTIVATE:
                if (typeof this.props.onActivate === 'function') {
                    this.props.onActivate();
                }
                return;
            case BarActionType.DEACTIVATE:
                if (typeof this.props.afterResizing === 'function') {
                    this.props.afterResizing();
                }
                return;
            default:
                return;
        }
    }
    refreshSizeInfos() {
        this.sizeRelatedInfoAction$.next(this.makeSizeInfos());
    }
    makeSizeInfos() {
        const { collect, getResult } = collectSizeRelatedInfo();
        this.childrenProps.forEach((childProps, index) => {
            const element = this.childrenInstance[index];
            collect({
                maxSize: childProps.maxSize,
                minSize: childProps.minSize,
                disableResponsive: isDisabledResponsive(childProps),
                isSolid: isSolid(childProps),
                currentSize: element.getBoundingClientRect()[this.dimension],
            });
        });
        return getResult();
    }
}
export { Container, Resizer };
